// This file is part of WifiCC3000 - the Managed Wifi Driver for the TI CC3000
// Hosted on Codeplex: http://cc3000.codeplex.com
// WifiCC3000 is free software licensed under the Apache License 2.0
//  Copyright 2012 ValkyrieTech, LLC

using System;
using Microsoft.SPOT;
using System.Text;

namespace Networking
{
    public static class DHCP
    {
        private static readonly byte[] MagicCookie = new byte[] { 0x63, 0x82, 0x53, 0x63 };
        
        public static byte[] ConstructInformMessage(byte[] IP, byte[] Mac, string Name)
        {
            var message = new byte[255 + Name.Length];

            message.Overwrite(0, new byte[] { 0x01, 0x01, 0x06 }); // Message Type, hardware type, add length
            message.Overwrite(4, Extensions.GetRandomBytes(4));  // Transaction ID
            //message.Overwrite(8, new byte[] { 0x03, 0x00 });  // Seconds elapsed
            message.Overwrite(10, new byte[] { 0x80 }); //bootp flags
            message.Overwrite(12, IP);
            message.Overwrite(28, Mac);
            message.Overwrite(236, MagicCookie);
            message.Overwrite(240, new byte[] { 0x35, 0x01, 0x08 }); // DHCP Inform
            message.Overwrite(243, new byte[] { 0x3d, 0x07, 0x01 }); // Client Identifier
            message.Overwrite(246, Mac);
            message.Overwrite(252, new byte[] { 0x0c, (byte)Name.Length });
            message.Overwrite(254, Encoding.UTF8.GetBytes(Name));  // Warning, Number of bytes and string length are not always equal in UTF8!!!

            message[254 + Name.Length] = 0xff;  //End 

            return message;
        }


    }
}
