// This file is part of WifiCC3000 - the Managed Wifi Driver for the TI CC3000
// Hosted on Codeplex: http://cc3000.codeplex.com
// WifiCC3000 is free software licensed under the Apache License 2.0
//  Copyright 2012 ValkyrieTech, LLC

using System;
using Microsoft.SPOT;
using System.Reflection;
using System.Collections;

namespace Networking
{
    public static class Extensions
    {
        public static void Overwrite(this byte[] origin, int start, byte[] source)
        {
            foreach (var aByte in source) origin[start++] = aByte;
        }

        public static void Overwrite(this byte[] origin, int start, byte[] source, int sourceLength)
        {
            int count = 0;

            foreach (var aByte in source)
            {
                origin[start++] = aByte;
                if (++count >= sourceLength) return;
            }
        }

        public static void AddRange(this ArrayList theList, Array values)
        {
            foreach (var aValue in values) theList.Add(aValue);
        }

        /// <summary>
        /// Finds the location of the pattern bytes within the buffer bytes. and returns   
        /// Returns a -1 if the pattern is not found within the buffer
        /// </summary>
        /// <param name="buffer">Bytes to be searched</param>
        /// <param name="pattern">Bytes you are looking for</param>
        /// <returns>Index of the first pattern byte within the buffer array</returns>
        public static int Locate(this byte[] buffer, byte[] pattern)
        {
            // Based on: http://stackoverflow.com/questions/283456/byte-array-pattern-search

            for (int i = 0; i < buffer.Length; i++)
            {
                if (pattern[0] == buffer[i] && buffer.Length - i >= pattern.Length)
                {
                    bool ismatch = true;
                    for (int j = 1; j < pattern.Length && ismatch == true; j++)
                    {
                        if (buffer[i + j] != pattern[j])
                        {
                            ismatch = false;
                            continue;
                        }
                    }

                    if (ismatch) return i;
                }
            }
            
            return -1;
        }

      


        static Random random = new Random();

        public static byte[] GetRandomBytes(short count)
        {
            byte[] buffer = new byte[count];
            random.NextBytes(buffer);
            return buffer;

            //string result = String.Concat(buffer.Select(x => x.ToString("X2")).ToArray());
            //if (digits % 2 == 0)
            //    return result;
            //return result + random.Next(16).ToString("X");
        }

        ///// <summary>
        ///// Calculate the IPv4 Checksum of the buffer array.  
        ///// </summary>
        ///// <param name="buffer"></param>
        ///// <param name="start"></param>
        ///// <param name="length"></param>
        ///// <returns></returns>
        //public static byte[] InternetChecksum(this byte[] buffer, int length = 0, int start = 0)
        //{
        //    // Based on: http://stackoverflow.com/questions/2188060/calculate-an-internet-aka-ip-aka-rfc791-checksum-in-c-sharp

        //    //byte[] buffer = value;
        //    length = length == 0 ? buffer.Length : length;
        //    int i = start;
        //    UInt32 sum = 0;
        //    UInt32 data = 0;
        //    while (length > 1)
        //    {
        //        data = 0;
        //        data = (UInt32)(((UInt32)(buffer[i]) << 8) | ((UInt32)(buffer[i + 1]) & 0xFF));
        //        sum += data;
                
        //        if ((sum & 0xFFFF0000) > 0)
        //        {
        //            sum = sum & 0xFFFF;
        //            sum += 1;
        //        }

        //        //Debug.WriteLine("Bytes: " + buffer[i].ToString() + ", " + buffer[i + 1].ToString());
        //        //Debug.WriteLine("Sum = " + sum + ", Length = " + length + ", i = " + i);
                
        //        i += 2;
        //        length -= 2;
        //    }

        //    if (length > 0)
        //    {
        //        sum += (UInt32)(buffer[i] << 8);
        //        if ((sum & 0xFFFF0000) > 0)
        //        {
        //            sum = sum & 0xFFFF;
        //            sum += 1;
        //        }
        //    }

        //    sum = ~sum;
        //    sum = sum & 0xFFFF;

        //    var result = new byte[2];

        //    result[0] = (byte)(sum >> 8);
        //    result[1] = (byte)sum;

        //   // result[0] = (byte)((UInt16)sum >> 8);
        //   //result[1] = (byte)(UInt16)sum;            

        //    return result;
        //}

        public static byte[] GetBroadcastAddress(this byte[] ipAdressBytes, byte[] subnetMaskBytes)
        {
            // based on: http://stackoverflow.com/questions/777617/calculate-broadcast-address-from-ip-and-subnet-mask

            //determines a broadcast address from an ip and subnet

            if (ipAdressBytes.Length != subnetMaskBytes.Length)
                throw new ArgumentException("Lengths of IP address and subnet mask do not match.");

            byte[] broadcastAddress = new byte[ipAdressBytes.Length];
            for (int i = 0; i < broadcastAddress.Length; i++)
            {
                broadcastAddress[i] = (byte)(ipAdressBytes[i] | (subnetMaskBytes[i] ^ 255));
            }
            return broadcastAddress;
        }

        /// <summary>
        /// Calculate the IPv4 Checksum of the buffer array.  
        /// </summary>
        /// <param name="buffer"></param>
        /// <param name="start"></param>
        /// <param name="length"></param>
        /// <returns></returns>
        public static byte[] InternetChecksum(this byte[] buffer, int length = 0, int start = 0, byte[] SourceIP = null, byte[] DestiantionIP = null, byte Protocol = 0x06)
        {
            // Based on: http://stackoverflow.com/questions/2188060/calculate-an-internet-aka-ip-aka-rfc791-checksum-in-c-sharp

            //byte[] buffer = value;
            length = length == 0 ? buffer.Length : length;
            int i = start;
            UInt32 sum = 0;
            UInt32 data = 0;
            byte[] pseudoHeader = null;

            if (SourceIP != null && DestiantionIP != null)
            {
                pseudoHeader = new byte[12] { SourceIP[0], SourceIP[1], SourceIP[2], SourceIP[3], DestiantionIP[0], DestiantionIP[1], DestiantionIP[2], DestiantionIP[3], 0x00, Protocol, (byte)(length >> 8), (byte)(length >> 0) };
                length += pseudoHeader.Length;
                i -= pseudoHeader.Length;
            }
                        
            while (length > 1)
            {
                //TODO: this is hideous and needs to be refactored!
                data = 0;
                if (i < start)
                    data = (UInt32)(((UInt32)(pseudoHeader[(i - start) + pseudoHeader.Length]) << 8) | ((UInt32)(pseudoHeader[(i - start) + pseudoHeader.Length + 1]) & 0xFF));                
                else
                    data = (UInt32)(((UInt32)(buffer[i]) << 8) | ((UInt32)(buffer[i + 1]) & 0xFF));
                
                sum += data;

                if ((sum & 0xFFFF0000) > 0)
                {
                    sum = sum & 0xFFFF;
                    sum += 1;
                }

                //Debug.WriteLine("Bytes: " + buffer[i].ToString() + ", " + buffer[i + 1].ToString());
                //Debug.WriteLine("Sum = " + sum + ", Length = " + length + ", i = " + i);

                i += 2;
                length -= 2;
            }

            if (length > 0)
            {
                sum += (UInt32)(buffer[i] << 8);
                if ((sum & 0xFFFF0000) > 0)
                {
                    sum = sum & 0xFFFF;
                    sum += 1;
                }
            }

            sum = ~sum;
            sum = sum & 0xFFFF;

            var result = new byte[2];

            result[0] = (byte)(sum >> 8);
            result[1] = (byte)sum;

            // result[0] = (byte)((UInt16)sum >> 8);
            //result[1] = (byte)(UInt16)sum;            

            return result;
        }

        ///// <summary>
        ///// Calculate the IPv4 Checksum of the buffer array.  
        ///// </summary>
        ///// <param name="buffer"></param>
        ///// <param name="start"></param>
        ///// <param name="length"></param>
        ///// <returns></returns>
        //public static byte[] InternetChecksum2(this byte[] buffer, int length = 0, int start = 0)
        //{
        //    // Based on: http://stackoverflow.com/questions/2188060/calculate-an-internet-aka-ip-aka-rfc791-checksum-in-c-sharp

        //    //byte[] buffer = value;
        //    length = length == 0 ? buffer.Length : length;
        //    int i = start;
        //    UInt32 sum = 0;
        //    while (i < length-1)
        //    {
        //        sum += (UInt32)(((UInt32)(buffer[i]) << 8) | ((UInt32)(buffer[i + 1]) & 0xFF));

        //        if ((sum & 0xFFFF0000) > 0) sum = sum++ & 0xFFFF;

        //        //Debug.WriteLine("Bytes: " + buffer[i].ToString() + ", " + buffer[i + 1].ToString());
        //        //Debug.WriteLine("Sum = " + sum + ", Length = " + length + ", i = " + i);

        //        i += 2;
        //    }

        //    if (length % 2 == 1)  
        //    {
        //        sum += (UInt32)(buffer[i] << 8);
        //        if ((sum & 0xFFFF0000) > 0) sum = sum++ & 0xFFFF;
        //    }

        //    sum = ~sum;
        //    sum = sum & 0xFFFF;

        //    var result = new byte[2];

        //    result[0] = (byte)(sum >> 8);
        //    result[1] = (byte)sum;

        //    // result[0] = (byte)((UInt16)sum >> 8);
        //    //result[1] = (byte)(UInt16)sum;            

        //    return result;
        //}


        ///// <summary>
        ///// Convert to a 8-byte unsigned integer
        ///// </summary>
        ///// <param name="array"></param>
        ///// <returns></returns>
        //public static ulong ToLong(this byte[] array, short offset = 0, short length = 8)
        //{
        //    int pos = System.Math.Min(length, array.Length) * 8;
        //    ulong result = 0;
            
        //    //foreach (byte by in array)
        //    for (short i = offset; i < System.Math.Min(length, array.Length); i++)
        //    {
        //        pos -= 8;
        //        result |= (ulong)array[i] << pos;
        //    }

        //    return result;
        //}

        /// <summary>
        /// Convert to a 8-byte unsigned integer
        /// </summary>
        /// <param name="array"></param>
        /// <returns></returns>
        public static ulong ToLong(this byte[] array, short offset = 0, short length = 8)
        {
            int pos = 0;
            ulong result = 0;

            //foreach (byte by in array)
            for (short i = offset; i < (length + offset); i++)
            {
                result |= (ulong)array[i] << pos;
                pos += 8;
            }

            return result;
        }


        /// <summary>
        /// Convert to a 4-byte unsigned integer
        /// </summary>
        /// <param name="array"></param>
        /// <returns></returns>
        public static uint ToInt(this byte[] array, short offset = 0)
        {
            return (uint)ToLong(array, offset, 4);
        }

        /// <summary>
        /// Convert to a 4-byte unsigned integer
        /// </summary>
        /// <param name="array"></param>
        /// <returns></returns>
        public static uint ToInt(this bool value)
        {
            return value ? 1u : 0u;
        }

        /// <summary>
        /// Convert to a 2-byte unsigned integer
        /// </summary>
        /// <param name="array"></param>
        /// <returns></returns>
        public static ushort ToShort(this byte[] array, short offset = 0)
        {
            return (ushort)ToLong(array, offset, 2);
        }

        public static byte[] ToBytes(this ushort num)
        {
            return new byte[] { (byte)(num >> 8), (byte)(num >> 0) };
        }

        public static byte[] ToBytes(this short num)
        {
            return new byte[] { (byte)(num >> 8), (byte)(num >> 0) };
        }

        public static byte[] ToBytes(this uint num)
        {
            return new byte[] { (byte)(num >> 24), (byte)(num >> 16), (byte)(num >> 8), (byte)(num >> 0) };
        }

        public static byte[] ToBytesRev(this ushort num)
        {
            return new byte[] { (byte)(num >> 0), (byte)(num >> 8) };
        }

        public static byte[] ToBytesRev(this short num)
        {
            return new byte[] { (byte)(num >> 0), (byte)(num >> 8) };
        }

        public static byte[] ToBytesRev(this uint num)
        {
            return new byte[] { (byte)(num >> 0), (byte)(num >> 8), (byte)(num >> 16), (byte)(num >> 24) };
        }

        public static byte[] ToBytesRev(this int num)
        {
            return new byte[] { (byte)(num >> 0), (byte)(num >> 8), (byte)(num >> 16), (byte)(num >> 24) };
        }

        private const string digits = "0123456789.-";

        public static bool IsNumeric(this string value)
        {
            foreach (var aChar in value)
                if (digits.IndexOf(aChar) == -1) return false;

            return true;
        }

        /// <summary>
        /// Validates and converts a string IP Address (like "192.168.1.1") to a byte array.
        /// </summary>
        /// <param name="ipAddress"></param>
        /// <returns>IP Address representation as bytes.  Returns null if validation fails.  </returns>
        public static byte[] ToBytes(this string ipAddress)
        {
            if (ipAddress.IndexOf('.') > 0)
            {
                var dnsParts = ipAddress.Split('.');

                // if the dnsname is already an IP address, just convert to bytes and return that!  
                if (dnsParts.Length == 4 && dnsParts[3].IsNumeric() && dnsParts[2].IsNumeric() && dnsParts[1].IsNumeric() && dnsParts[0].IsNumeric())
                    return new byte[4] { (byte)(ushort.Parse(dnsParts[0]) >> 0), (byte)(ushort.Parse(dnsParts[1]) >> 0), (byte)(ushort.Parse(dnsParts[2]) >> 0), (byte)(ushort.Parse(dnsParts[3]) >> 0) };
            }

            return null;
        }

        const string hexDigits = "0123456789ABCDEF";

        public static string ToHexString(this byte[] buffer, short offset = 0)
        {
            // Based on: http://stackoverflow.com/questions/311165/how-do-you-convert-byte-array-to-hexadecimal-string-and-vice-versa-in-c

            if (buffer == null) return string.Empty;

            var chars = new char[(buffer.Length - offset) * 2];

            for (short y = offset, x = offset; y < (buffer.Length - offset); ++y, ++x)
            {
                chars[x] = hexDigits[(buffer[y] & 0xF0) >> 4];
                chars[++x] = hexDigits[(buffer[y] & 0x0F)];
            }

            return new string(chars);
        }

        public static string ToHexString(this short value)
        {
            return value.ToBytes().ToHexString();
        }

        public static string ToHexString(this ushort value)
        {
            return value.ToBytes().ToHexString();
        }

        
        /// <summary>
        /// Standard dot separated integers, like 192.168.1.255
        /// </summary>
        /// <param name="array"></param>
        /// <returns></returns>
        public static string ToAddress(this byte[] array)
        {
            string result = string.Empty;
            
            if (array != null)
                foreach (byte aByte in array)
                    result += ((uint)aByte).ToString() + ".";

            return result.TrimEnd('.');
        }

        public static bool BytesEqual(this byte[] Array1, byte[] Array2)
        {
            return Array1.BytesEqual(0, Array2, 0, Array1.Length);
        }

        public static bool BytesEqual(this byte[] Array1, int Start1, byte[] Array2, int Start2, int Count)
        {
            bool result = true;
            for (int i = 0; i < Count; i++)
            {
                if (Array1[i + Start1] != Array2[i + Start2])
                {
                    result = false;
                    break;
                }
            }
            return result;
        }

    }
}

//// This code allows Extension methods to work in .NET MF 4.1
//namespace System.Runtime.CompilerServices
//{
//    [AttributeUsage(AttributeTargets.Method, AllowMultiple = false, Inherited = false)]
//    public class ExtensionAttribute : Attribute
//    {
//    }
//}
