// This file is part of WifiCC3000 - the Managed Wifi Driver for the TI CC3000
// Hosted on Codeplex: http://cc3000.codeplex.com
// WifiCC3000 is free software licensed under the Apache License 2.0
//  Copyright 2012 ValkyrieTech, LLC


using System;
//using Microsoft.SPOT;
using Microsoft.SPOT.Hardware;
using System.Diagnostics;
using Microsoft.SPOT;

namespace Networking
{
    /// <summary>
    /// Handles on the Naming part of Netbios over TCP
    /// </summary>
    public static class NetBiosNaming
    {
        //private static byte[] scratch = new byte[104] { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x08, 0x00, 0x45, 0x00, 0x00, 0x5a, 0x00, 0x05, 0x00, 0x00, 0xff, 0x11, 0x37, 0x8d, 0xc0, 0xa8, 0x01, 0x5a, 0xc0, 0xa8, 0x01, 0x56, 0x00, 0x89, 0x00, 0x89, 0x00, 0x46, 0x00, 0x00, 0xdd, 0x1a, 0x85, 0x00, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x00, 0x20, 0x45, 0x4f, 0x45, 0x46, 0x46, 0x45, 0x45, 0x45, 0x46, 0x46, 0x45, 0x4a, 0x45, 0x4f, 0x45, 0x50, 0x43, 0x41, 0x43, 0x41, 0x43, 0x41, 0x43, 0x41, 0x43, 0x41, 0x43, 0x41, 0x43, 0x41, 0x41, 0x41, 0x00, 0x00, 0x20, 0x00, 0x01, 0x00, 0x00, 0x0f, 0x0f, 0x00, 0x06, 0x60, 0x00, 0xc0, 0xa8, 0x01, 0x5a };

        public static bool ContainsNameRequestFor(this byte[] payload, string name)
        {
            Debug.Assert(name.IndexOf(".local") > 0, "Local names must end with .local");

            byte[] nbName = new byte[32];
                
            //TODO: This is bad!  It does not really parse the answers and look for the name.  It assumes a fixed position, which is bad!  
            Array.Copy(payload, 13, nbName, 0, 32);
            Debug.Print("Netbios name query for: " + DecodeNetbiosName(nbName));

            // Flexible NetBios Name matching
            return (DecodeNetbiosName(nbName).Trim().ToLower() == name || DecodeNetbiosName(nbName).Trim().ToLower() == name.Split('.')[0]);
        }

        public static byte[] ParseTransactionID(this byte[] payload)
        {
            return new byte[] { payload[0], payload[1] };
        }


        /// <summary>
        /// Netbios Name Reply
        /// </summary>
        public static byte[] ConstructNetbiosResponse(string name, byte[] IP, byte[] transactionID)
        {
            if (name.IndexOf('.') > 0) name = name.Split('.')[0];  // trim off the .local if it exists because NBNS does not like it!  
            
            var nbName = EncodeNetbiosName(name);
            var message = new byte[nbName.Length + 30];

            message.Overwrite(0, transactionID);
            message[2] = 0x85;  // Flags
            message[7] = 0x01;  // Answer RRs

            // Answer
            message[12] = 0x20;  // I think this is the name length, which is fixed to 32 (20h)
            message.Overwrite(13, nbName);
            var offset = nbName.Length + 13;
            message[offset + 2] = 0x20;  // Type NB
            message[offset + 4] = 0x01;  // Class IN
//            message.Overwrite(offset + 5, new byte[] { 0x00, 0x00, 0x0F, 0x0F });  // Time to Live
            message.Overwrite(offset + 5, new byte[] { 0x00, 0x00, 0x00, 0x1F });  // Time to Live
            message[offset + 10] = 0x06;  // data length ??
            message[offset + 11] = 0x60; // Flags
            message.Overwrite(offset + 13, IP);

            return message;
        }

        static byte[] EncodeNetbiosName(string Name)
        {
            byte[] result = new byte[32];
            char c;

            Name = Name.ToUpper();

            for (int i = 0; i < 15; i++)
            {
                c = i < Name.Length ? Name[i] : ' ';
                result[i * 2] = (byte)(((byte)(c) >> 4) + 65);
                result[(i * 2) + 1] = (byte)(((byte)(c) & 0x0f) + 65);
            } 
            
            result[30] = 0x41;
            result[31] = 0x41;

            return result;
        }

        static string DecodeNetbiosName(byte[] NbName)
        {
            string result = string.Empty;
            for (int i = 0; i < 15; i++)
            {
                byte b1 = NbName[i * 2];
                byte b2 = NbName[(i * 2) + 1];
                char c = (char)(((b1 - 65) << 4) | (b2 - 65));
                result += c;
            }

            return result;
        }

    }
}
