// This file is part of WifiCC3000 - the Managed Wifi Driver for the TI CC3000
// Hosted on Codeplex: http://cc3000.codeplex.com
// WifiCC3000 is free software licensed under the Apache License 2.0
//  Copyright 2012 ValkyrieTech, LLC

using System;
using Microsoft.SPOT;
using Microsoft.SPOT.Hardware;
using Networking;
using System.Text;

namespace WifiExampleApp
{
    public class InternetTimeExample
    {
        static UdpSocket s = null;
        static bool waiting = true;

        public static void Main()
        {
            Debug.Print("CC3000 Internet Time Example");

            var wifi = new WifiAdapter("CC3000", InterfaceProfile.VTech_Device2);
            wifi.Connected += wifi_Connected;

            // Manually specify a network
            //var ap = new AccessPoint() { Ssid = "linksys", Key = "password", Security = WifiSecurityType.WPA2 };
            //wifi.Connect(ap);

            while (waiting) System.Threading.Thread.Sleep(50);

            wifi.Stop();  // turns off power to wifi adapter
        }

        static void wifi_Connected(WifiAdapter wifiAdapter)
        {
            Debug.Print("Woot!  We are now connected to Wifi!");
            Debug.Print("Network Name: " + wifiAdapter.SsidName);
            Debug.Print("IP address: " + wifiAdapter.IPAddress.ToAddress());
            Debug.Print("MacAddress: " + wifiAdapter.MacAddress.ToHexString());
            Debug.Print("Gateway: " + wifiAdapter.Gateway.ToAddress());
            Debug.Print("DNS Server: " + wifiAdapter.DomainNameServer.ToAddress());

            s = wifiAdapter.OpenUdpSocket();
            
            // Listen for UDP messages sent to activated ports
            s.OnDataReceived += s_OnDataReceived;
            
            // Activate the NTP (date/time) port 123
            s.Listen(123);

            // Create a NTP (date/time) Request Message
            var msg = new byte[48];
            msg[0] = 0x1b;

            // Let's get the UTC time from a time zerver using a UDP Message
            s.Send(msg, new byte[4] { 0x40, 0x5a, 0xb6, 0x37 }, 123);  // 64.90.182.55 the address of a NIST time server
            //s.Send(msg, new byte[4] { 0xD8, 0xAB, 0x78, 0x24 }, 123);  // 216.171.120.36 chicago NTP server
            //TODO: do a lookup for time.nist.gov first...

            while (waiting) System.Threading.Thread.Sleep(100);

            s.StopListening();

            s.OnDataReceived -= s_OnDataReceived;
        }

        static void s_OnDataReceived(UdpSocket sender, byte[] message, byte[] remoteIP, ushort remotePort)
        {
            if (remotePort == 123)
            {
                var transitTime = Utility.ExtractRangeFromArray(message, 40, 8);
                Microsoft.SPOT.Debug.Print("Current UTC Date/Time is " + transitTime.ToDateTime());

                waiting = false;
            }
        }
    }

    public static class Extensions
    {
        /// <summary>
        /// Convert an 8-byte array from NTP format to .NET DateTime.  
        /// </summary>
        /// <param name="ntpTime">NTP format 8-byte array containing date and time</param>
        /// <returns>A Standard .NET DateTime</returns>
        public static DateTime ToDateTime(this byte[] ntpTime)
        {
            Microsoft.SPOT.Debug.Assert(ntpTime.Length == 8, "The passed array is too short to be a valid ntp date and time");

            ulong intpart = 0;
            ulong fractpart = 0;

            for (int i = 0; i <= 3; i++)
                intpart = (intpart << 8) | ntpTime[i];

            for (int i = 4; i <= 7; i++)
                fractpart = (fractpart << 8) | ntpTime[i];

            ulong milliseconds = (intpart * 1000 + (fractpart * 1000) / 0x100000000L);

            var timeSince1900 = TimeSpan.FromTicks((long)milliseconds * TimeSpan.TicksPerMillisecond);
            return new DateTime(1900, 1, 1).Add(timeSince1900);

        }
    }
}


