// This file is part of WifiCC3000 - the Managed Wifi Driver for the TI CC3000
// Hosted on Codeplex: http://cc3000.codeplex.com
// WifiCC3000 is free software licensed under the Apache License 2.0
//  Copyright 2012 ValkyrieTech, LLC

using System;
using Microsoft.SPOT;
using Microsoft.SPOT.Hardware;
using Networking;
using System.Text;
using System.Threading;

namespace WifiExampleApp
{
    public class WebServerWithNameExample
    {
        static TcpSocket webServer = null;
        static UdpSocket llmnr = null;
        static UdpSocket mdns = null;
        static UdpSocket nbns = null;

        static WifiAdapter wifi;

        public static void Main()
        {
            Debug.Print("CC3000 Web Server with Local Name resolution Example Application");

            wifi = new WifiAdapter("cc3000", InterfaceProfile.VTech_Device2);
            wifi.Connected += wifi_Connected;

            // Manually specify a network
            //var ap = new AccessPoint() { Ssid = "linksys", Key = "password", Security = WifiSecurityType.WPA2 };
            //wifi.Connect(ap);

            while (true) System.Threading.Thread.Sleep(5);
        }

        static void wifi_Connected(WifiAdapter wifiAdapter)
        {
            Debug.Print("Woot!  We are now connected to Wifi!");
            Debug.Print("Network Name: " + wifiAdapter.SsidName);
            Debug.Print("IP address: " + wifiAdapter.IPAddress.ToAddress());
            //Debug.Print("MacAddress: " + wifiAdapter.MacAddress.ToHexString());
            Debug.Print("Gateway: " + wifiAdapter.Gateway.ToAddress());
            Debug.Print("DNS Server: " + wifiAdapter.DomainNameServer.ToAddress());

            if (wifi.Name != null && wifi.Name.Trim() != string.Empty)
            {
                // Listen for MDNS packets
                if (mdns == null)  // check for null so we don't open new sockets on a reconnect
                {
                    mdns = wifiAdapter.OpenUdpSocket();
                    mdns.PollFrequency = 400;
                    mdns.OnDataReceived += mdns_OnDataReceived;
                    mdns.Listen(5353, new byte[] { 224, 0, 0, 251 });  // MDNS
                }

                // Listen for NetBios Name requests
                if (nbns == null)  // check for null so we don't open new sockets on a reconnect
                {
                    nbns = wifiAdapter.OpenUdpSocket();
                    nbns.PollFrequency = 400;
                    nbns.OnDataReceived += nbns_OnDataReceived;
                    nbns.Listen(137);  // Netbios Port
                }
            }

            // Web Server Listener
            if (webServer == null) // check for null so we don't open new sockets on a reconnect
            {
                webServer = wifiAdapter.OpenTcpSocket();
                webServer.OnDataReceived += s_OnDataReceived;
                webServer.Listen(80);
            }
        }

        static void nbns_OnDataReceived(UdpSocket sender, byte[] message, byte[] remoteIP, ushort remotePort)
        {
            Debug.Print("NetBIOS Message Detected from IP: " + remoteIP.ToAddress());

            if (!remoteIP.BytesEqual(wifi.IPAddress) && remotePort == 137 && message.ContainsNameRequestFor(wifi.Name))
            {
                byte[] response = NetBiosNaming.ConstructNetbiosResponse(wifi.Name, wifi.IPAddress, message.ParseTransactionID());
                Debug.Print("Sending NetBIOS Response!");
                sender.Send(response, wifi.IPAddress.GetBroadcastAddress(wifi.SubnetMask) , 137);
            }
        }

        static void mdns_OnDataReceived(UdpSocket sender, byte[] message, byte[] remoteIP, ushort remotePort)
        {
            if (remoteIP.BytesEqual(wifi.IPAddress)) return;  // do not respond to our own MDNS messages!  
            
            Debug.Print("Multicast MDNS UDP Message Received (" + message.Length + " bytes) from " + remoteIP.ToAddress() + ":" + remotePort);
            
            if (message.ContainsMDnsNameRequestFor(wifi.Name)) 
            {
                //byte[] mdnsResponse = Networking.DNS.ConstructDnsResponse(wifi.Name, new byte[2] { message[0], message[1] }, wifi.IPAddress);
                byte[] mdnsResponse = Networking.DNS.ConstructMDnsResponse(wifi.Name, wifi.IPAddress, 200);

                Debug.Print("Sending MDNS Response!");
                sender.Send(mdnsResponse, new byte[] { 224, 0, 0, 251 }, 5353);
            }
        }

        static void s_OnDataReceived(TcpSocket sender, byte[] message, byte[] remoteIP, ushort remotePort)
        {
            Debug.Print("New Message Received: " + new string(Encoding.UTF8.GetChars(message)));

            var body = "<html><head><meta http-equiv=\"refresh\" content=\"10\"></head><body><font face=\"verdana\"><h1>The CC3000 Managed Driver</h1><p>for .NET MicroFramework</p><p>This is just the beginning :)</p><p><a href=\"http://cc3000.codeplex.com\">Visit us at Codeplex!</a></p></font></body></html>";

            var response = new HttpResponse(body);
            //response.Connection = "Close";

            sender.Send(response.HeaderSection);
            sender.Send(Encoding.UTF8.GetBytes(body));

            //sender.Close();
        }
    }
}
